#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>
#include "funcs.h"
#include "config.h"
#include "db.h"
#include "builder.h"

struct CONFIG cfg;

void help(void) {
	////

	printf("%s -[f:d:h] [add/rebuild/version/del]:\n\tadd [img] [tag1] [tag2]...\n\trebuild\n\tversion\n\tdel [image id]\n\nOptions:\n\t-d STR set description\n\t-f STR Path to config file\n\t-h This menu\n", cfg.prog_name);
	exit(0);
}

char *gen_id(const char *filename) {
	const char *alf = "qwertyuiopasdfghjklzxcvbnm1234567890QWERTYUIOPASDFGHJKLZXCVBNM";

	char *ext = strrchr(filename, '.');
	if (ext == NULL)
		ext = "";

	size_t size = 7 + strlen(ext);
	char *id = malloc(size);
	if (id == NULL) {
		fprintf(stderr, "%s: malloc: %s\n", cfg.prog_name, strerror(errno));
		return NULL;
	}

	for (size_t i = 0; i < size - strlen(ext); i++)
	id[i] = alf[rand() % sizeof(alf)];

	strcat(id, ext);
	id[size] = '\0';
	return id;
}

int copy(const char *src, const char *dst) {
	FILE *ifp = file_open(cfg.prog_name, src, "r");
	if (ifp == NULL)
		return 1;

	FILE *ofp = file_open(cfg.prog_name, dst, "w");
	if (ofp == NULL) {
		fclose(ifp);
		return 1;
	}

	char buf[4096];
	size_t ret = 0;
	while ((ret = fread(buf, sizeof(char), sizeof(buf), ifp)))
		fwrite(buf, sizeof(char), ret, ofp);

	fclose(ifp);
	fclose(ofp);
	return 0;
}

int add(char **list, const int size, const char *desc) {
	char *name = NULL;

	/* Generate new filename (image id) */
	while (1) {
		name = gen_id(list[0]);
		if (name == NULL)
			return 1;

		if (access(name, W_OK))
			break;

		free(name);
	}

	/* Copy file */
	if (copy(list[0], name)) {
		free(name);
		return 1;
	}

	/* Update db */
	FILE *fp = file_open(cfg.prog_name, "db", "a");
	if (fp == NULL) {
		free(name);
		return 1;
	}

	/* Write "filename|tag1,tag2...|desc" to the db */
	fprintf(fp, "%s|", name);
	if (size > 1) {
		for (int i = 1; i < size; i++) {
			if (strchr(list[i], ',') == NULL || strchr(list[i], ' ') == NULL)
				fprintf(fp, "%s%c", list[i], (i == size - 1) ? '|' : ',');

			else
				fprintf(stderr, "%s: tag '%s' contain ',', ' ' or '|'\n", cfg.prog_name, list[i]);
		}
	}

	else
		fputs("no_context|", fp);

	if (desc)
		fputs(desc, fp);

	fputc('\n', fp);
	free(name);
	fclose(fp);
	return 0;
}

int del(const char *str) {
	struct DB_STR *db = read_db(cfg);
	if (db == NULL)
		return 1;

	FILE *out = file_open(cfg.prog_name, "db", "w");
	if (out == NULL) {
		free_db(db);
		return 1;
	}

	struct DB_STR *bckp = db;
	while (bckp != NULL) {
		struct DB_STR *ptr = bckp->next;

		if (strstr(bckp->filename, str)) {
			remove(bckp->filename);

			bckp = ptr;
			continue;
		}

		fprintf(out, "%s|", bckp->filename);
		for (size_t i = 0; i < bckp->size; i++)
			fprintf(out, "%s%c", bckp->tags[i], (i == bckp->size - 1) ? '|' : ',');

		fprintf(out, "%s\n", bckp->desc);
		bckp = ptr;
	}

	free_db(db);
	fclose(out);

	return 0;
}

int main(int argc, char **argv) {
	srand(getpid());

	char *desc = NULL;
	cfg.prog_name = argv[0];

	/* Default config */
	cfg.root = ROOT;

	cfg.css = CSS;
	cfg.logo = LOGO;
	cfg.desc = DESC;
	cfg.title = TITLE;
	cfg.posts_at_page = POST_AT_PAGE;

	cfg.link = LINK;
	cfg.xml_file = XML_FILE;
	cfg.language = LANGUAGE;
	cfg.max_posts = MAX_POSTS;

	cfg.date_fmt = DATE_FORMAT;

	/* Parse arguments */
	int opt;
	while ((opt = getopt(argc, argv, "f:d:h")) != -1) {
		switch (opt) {
			case 'd':
				desc = optarg;
				break;

			case 'f':
				if (parse_config(&cfg, optarg))
					return 1;

				break;

			case 'h':
			default:
				help();
		}
	}

	argv += optind;
	argc -= optind;

	if (chdir(cfg.root) < 0) {
		fprintf(stderr, "%s: chdir: %s\n", cfg.prog_name, strerror(errno));

		////
		return 1;
	}

	int ret = 0;

	/* Parse args */
	if (!*argv)
		help();

	else if (argc > 1 && !strcmp(argv[0], "add"))
		ret = add(argv + 1, argc - 1, desc);

	else if (argc > 1 && !strcmp(argv[0], "del"))
		ret = del(*(argv + 1));

	else if (!strcmp(argv[0], "rebuild"))
		ret = rebuild(cfg);

	else if (!strcmp(argv[0], "version"))
		printf("%s (8img) version: 2\nWritten under WTFPL License.\n", cfg.prog_name);

	else
		help();

	////
	return ret;
}

